## This concern has +like+, +unlike+ controller actions
## When you use it, be sure that +set_sociable+ method is implemented in your controller, otherwise an fatal error will raise.
module SociableController
  extend ActiveSupport::Concern

  VIEWABLE_METHODS = [:show]

  included do
    before_action :authenticate_user!, only: [:like]
    before_action :authorize_sociable!, only: [:like]
    before_action :view_object!, only: VIEWABLE_METHODS
  end

  # PUT /v1/learning_objects/1/like
  # PUT /v1/learning_objects/1/like.json
  def like
    if !sociable.liked? current_user
      sociable.like current_user
      render json: {count: sociable.likes.count}, status: :created
    elsif sociable.liked? current_user
      sociable.dislike current_user
      render json: {count: sociable.likes.count}, status: :ok
    else
      render status: :forbidden
    end
  end

  protected

  def authorize_sociable!
    authorize sociable
  end

  def sociable
    raise NotImplementedError
  end

  def view_object!
    sociable.view current_user if user_signed_in?
  end

end
